@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2021 Joseph Gaeddert
+ * Copyright (c) 2007 - 2022 Joseph Gaeddert
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -33,19 +33,19 @@
 // include proper SIMD extensions for x86 platforms
 // NOTE: these pre-processor macros are defined in config.h
 
-#if HAVE_MMX && HAVE_MMINTRIN_H
+#if HAVE_MMX
 #include <mmintrin.h>   // MMX
 #endif
 
-#if HAVE_SSE && HAVE_XMMINTRIN_H
+#if HAVE_SSE
 #include <xmmintrin.h>  // SSE
 #endif
 
-#if HAVE_SSE2 && HAVE_EMMINTRIN_H
+#if HAVE_SSE2
 #include <emmintrin.h>  // SSE2
 #endif
 
-#if HAVE_SSE3 && HAVE_PMMINTRIN_H
+#if HAVE_SSE3
 #include <pmmintrin.h>  // SSE3
 #endif
 
@@ -173,6 +173,29 @@ dotprod_cccf dotprod_cccf_recreate_rev(dotprod_cccf    _q,
     return dotprod_cccf_create_rev(_h,_n);
 }
 
+dotprod_cccf dotprod_cccf_copy(dotprod_cccf q_orig)
+{
+    // validate input
+    if (q_orig == NULL)
+        return liquid_error_config("dotprod_cccf_copy().mmx, object cannot be NULL");
+
+    dotprod_cccf q_copy = (dotprod_cccf)malloc(sizeof(struct dotprod_cccf_s));
+    q_copy->n = q_orig->n;
+
+    // allocate memory for coefficients, 16-byte aligned (repeated)
+    q_copy->hi = (float*) _mm_malloc( 2*q_copy->n*sizeof(float), 16 );
+    q_copy->hq = (float*) _mm_malloc( 2*q_copy->n*sizeof(float), 16 );
+
+    // copy coefficients array (repeated)
+    //  hi = { crealf(_h[0]), crealf(_h[0]), ... crealf(_h[n-1]), crealf(_h[n-1])}
+    //  hq = { cimagf(_h[0]), cimagf(_h[0]), ... cimagf(_h[n-1]), cimagf(_h[n-1])}
+    memmove(q_copy->hi, q_orig->hi, 2*q_orig->n*sizeof(float));
+    memmove(q_copy->hq, q_orig->hq, 2*q_orig->n*sizeof(float));
+
+    // return object
+    return q_copy;
+}
+
 int dotprod_cccf_destroy(dotprod_cccf _q)
 {
     _mm_free(_q->hi);
@@ -245,7 +268,7 @@ int dotprod_cccf_execute_mmx(dotprod_cccf    _q,
     // aligned output array
     float w[4] __attribute__((aligned(16))) = {0,0,0,0};
 
-#if HAVE_SSE3 && HAVE_PMMINTRIN_H
+#if HAVE_SSE3
     // SSE3
     __m128 s;   // dot product
     __m128 sum = _mm_setzero_ps(); // load zeros into sum register
@@ -276,7 +299,7 @@ int dotprod_cccf_execute_mmx(dotprod_cccf    _q,
         // shuffle values
         cq = _mm_shuffle_ps( cq, cq, _MM_SHUFFLE(2,3,0,1) );
         
-#if HAVE_SSE3 && HAVE_PMMINTRIN_H
+#if HAVE_SSE3
         // SSE3: combine using addsub_ps()
         s = _mm_addsub_ps( ci, cq );
 
@@ -297,7 +320,7 @@ int dotprod_cccf_execute_mmx(dotprod_cccf    _q,
 #endif
     }
 
-#if HAVE_SSE3 && HAVE_PMMINTRIN_H
+#if HAVE_SSE3
     // unload packed array
     _mm_store_ps(w, sum);
 #endif
