@@ -33,19 +33,19 @@
 // include proper SIMD extensions for x86 platforms
 // NOTE: these pre-processor macros are defined in config.h
 
-#if HAVE_MMX && HAVE_MMINTRIN_H
+#if HAVE_MMX
 #include <mmintrin.h>   // MMX
 #endif
 
-#if HAVE_SSE && HAVE_XMMINTRIN_H
+#if HAVE_SSE
 #include <xmmintrin.h>  // SSE
 #endif
 
-#if HAVE_SSE2 && HAVE_EMMINTRIN_H
+#if HAVE_SSE2
 #include <emmintrin.h>  // SSE2
 #endif
 
-#if HAVE_SSE3 && HAVE_PMMINTRIN_H
+#if HAVE_SSE3
 #include <pmmintrin.h>  // SSE3
 #endif
 
@@ -79,7 +79,7 @@ float liquid_sumsqf(float *      _v,
     // aligned output array
     float w[4] __attribute__((aligned(16)));
 
-#if HAVE_SSE3 && HAVE_PMMINTRIN_H
+#if HAVE_SSE3
     // fold down into single value
     __m128 z = _mm_setzero_ps();
     sum = _mm_hadd_ps(sum, z);
